@@ -98,12 +98,16 @@ file_sel_response_cb (GtkWidget    *widget,
 	GSList         *selections, *iter;
 	GList          *item_list = NULL;
 
-
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (data->dialog);
 		return TRUE;
 	}
 
+	if (response == GTK_RESPONSE_HELP) {
+		show_help_dialog (GTK_WINDOW (data->dialog), "file-roller-add-options");
+		return TRUE;
+	}
+
 	current_folder = gtk_file_chooser_get_current_folder (file_sel);
 
 	/* check folder permissions. */
@@ -297,7 +301,7 @@ add_folder_cb (GtkWidget *widget,
 	DialogData  *data;
 	gchar       *dir;
 	GtkWidget   *main_box;
-	GtkWidget   *vbox, *hbox;
+	GtkWidget   *vbox;
 	GtkWidget   *table;	
 
 	data = g_new0 (DialogData, 1);
@@ -312,10 +316,9 @@ add_folder_cb (GtkWidget *widget,
 		gtk_file_chooser_dialog_new (_("Add a Folder"),
 					     GTK_WINDOW (data->window->app),
 					     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-					     GTK_STOCK_CANCEL, 
-					     GTK_RESPONSE_CANCEL,
-					     FR_STOCK_ADD, 
-					     GTK_RESPONSE_OK,
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     FR_STOCK_ADD, GTK_RESPONSE_OK,
+					     GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 					     NULL);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_sel), FALSE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
@@ -348,6 +351,10 @@ add_folder_cb (GtkWidget *widget,
 
 	gtk_box_pack_start (GTK_BOX (vbox), data->add_if_newer_checkbutton, 
 			    TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), data->include_subfold_checkbutton,
+			    TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), data->exclude_symlinks,
+			    TRUE, TRUE, 0);
 
 	table = gtk_table_new (2, 2, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
@@ -380,15 +387,6 @@ add_folder_cb (GtkWidget *widget,
 			  GTK_FILL|GTK_EXPAND, 0,
 			  0, 0);
 
-	gtk_box_pack_start (GTK_BOX (vbox), data->include_subfold_checkbutton,
-			    TRUE, TRUE, 0);
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_symlinks,
-			    TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
-
 	/**/
 	
 	vbox = gtk_vbox_new (FALSE, 5);
